# oidc <!-- omit from toc -->

[![Pub Version][pub_badge]][pub_link]
[![last commit][github_last_commit_image]][github_link]
[![codecov][coverage_badge]][coverage_link]
[![style: very good analysis][very_good_analysis_badge]][very_good_analysis_link]
[![License: MIT][license_badge]][license_link]


An [OpenId Connect RP (Relying Party)][spec_link] plugin for flutter.

Make sure you read the [![Wiki](https://img.shields.io/badge/wiki-purple)](https://bdaya-dev.github.io/oidc/) for extra information.

## Table Of Contents <!-- omit from toc -->

- [Introduction ✨](#introduction-)
- [Installation 💻](#installation-)
- [Usage 🛠️](#usage-️)
- [Features 📚](#features-)
  - [📜 Conformance](#-conformance)
  - [Implemented specs](#implemented-specs)
  - [WIP Specs](#wip-specs)


## Introduction ✨ 

This federated plugin builds on top of [![oidc_core][oidc_core_image]][oidc_core_link] to add platform-specific handling which is required by the spec (e.g. launching a browser, listening for redirect, etc...).

## Installation 💻

**❗ In order to start using this plugin you must have the [Flutter SDK][flutter_install_link] installed on your machine.**

Add to your `pubspec.yaml`:

```sh
dart pub add oidc oidc_default_store
```

## Usage 🛠️

After following the [Getting Started](https://bdaya-dev.github.io/oidc/oidc-getting-started/) steps, it's as easy as:

```dart
//1. create the manager:
final manager = OidcUserManager.lazy(
    discoveryDocumentUri: OidcUtils.getOpenIdConfigWellKnownUri(
        Uri.parse('https://server.example.com'),
    ),
    // TODO: add other settings
);

//2. init()
await manager.init();

//3. listen to user changes
manager.userChanges().listen((user) {
  print('currentUser changed to $user');
});

//4. login
final newUser = await manager.loginAuthorizationCodeFlow();

//5. logout
await manager.logout();
```

## Features 📚

- 🧩 **Cross platform**: most features work on all platforms that can run flutter (Android, Ios, macos, web, windows, linux).
- 🧰 **High maintenance**: everyone hates having to fix an unmaintained package. you can trust that we will solve issues as soon as they pop up. especially since we use this package in all our production apps.
- ⚙️ **Customizability**: you can customize everything; Where to store the data, provide your own http client, extend requests/responses with your own data; whatever you want, you can do.
- 🚀 **Easy to use**: you mainly need to concern yourself with the `OidcUserManager` class, which is very well documented and has a simple interface.

### 📜 Conformance
- while we are **NOT** openid certified *yet*, this package was tested against [multiple conformance profiles](https://github.com/Bdaya-Dev/oidc/issues/11).

### Implemented specs

- [OpenId Connect Core 1.0][oidc_core_spec_link].
    - no hybrid flow support yet.    
- [OpenId Connect Discovery][discovery_spec_link].
    - no webfinger support yet.
- [RP Initiated logout][rp_logout_link].
- [Front-Channel Logout][frontchannel_logout_link].
- [Authorization code grant][auth_code_link] with [PKCE](pkce_link).
- [Resource Owner Password Credentials Grant](https://www.rfc-editor.org/rfc/rfc6749#section-1.3.3).
- Automatic [Refresh Token](https://oauth.net/2/grant-types/refresh-token/) rotation.
- [OAuth 2.0 For Native Apps](https://datatracker.ietf.org/doc/html/rfc8252)
 
### WIP Specs

- [Dynamic Client Registration](https://openid.net/specs/openid-connect-registration-1_0.html)
- [Session Management](https://openid.net/specs/openid-connect-session-1_0.html)


--- 
Plugin Generated by the [Very Good CLI][very_good_cli_link] 🤖

[flutter_install_link]: https://docs.flutter.dev/get-started/install

[github_link]: https://github.com/Bdaya-Dev/oidc
[github_last_commit_image]: https://img.shields.io/github/last-commit/bdaya-dev/oidc/main
[oidc_core_link]: https://pub.dev/packages/oidc_core
[oidc_core_image]: https://img.shields.io/badge/package-oidc__core-0175C2?logo=dart&logoColor=white

[spec_link]: https://openid.net/wg/connect/specifications/
[oidc_core_spec_link]: https://openid.net/specs/openid-connect-core-1_0.html
[auth_code_link]: https://oauth.net/2/grant-types/authorization-code/
[pkce_link]: https://datatracker.ietf.org/doc/html/rfc7636
[discovery_spec_link]: https://openid.net/specs/openid-connect-discovery-1_0.html
[rp_logout_link]: https://openid.net/specs/openid-connect-rpinitiated-1_0.html
[frontchannel_logout_link]: https://openid.net/specs/openid-connect-frontchannel-1_0.html

[coverage_link]: https://codecov.io/github/Bdaya-Dev/oidc
[coverage_badge]: https://codecov.io/github/Bdaya-Dev/oidc/graph/badge.svg?token=HSEDM6I7TH

[pub_badge]: https://img.shields.io/pub/v/oidc
[pub_link]: https://pub.dev/packages/oidc

[license_badge]: https://img.shields.io/badge/license-MIT-blue.svg
[license_link]: https://opensource.org/licenses/MIT

[very_good_analysis_badge]: https://img.shields.io/badge/style-very_good_analysis-B22C89.svg
[very_good_analysis_link]: https://pub.dev/packages/very_good_analysis

[very_good_cli_link]: https://github.com/VeryGoodOpenSource/very_good_cli